package hero.hook;

import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.Constants;
import hero.util.HeroHookException;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.objectweb.petals.binding.rmi.client.RmiServiceClientFactory;
import org.objectweb.petals.binding.rmi.model.MessageExchangeBean;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class PetalsAutomaticHook extends PetalsHook implements NodeHookI {
    
    public String getMetadata() {
        return Constants.Nd.AFTERTERMINATE;
    }

    @SuppressWarnings("unchecked")
    public void afterTerminate(Object engineSession, BnNodeLocal activity) throws HeroHookException {
        System.out.println("afterTerminate start");
        System.out.println("node name: " + activity.getName());
        
        try {
            Map<String, String> projectProperties = getProjectPropertiesMap(activity.getBnProject().getBnProperties());
            Map<String, String> activityProperties = getActivityPropertiesMap(activity.getBnProperties());
            
            MessageExchangeBean msgExchange = new MessageExchangeBean();
            
            String service = getService(activityProperties);
            String operation = getOperation(activityProperties);

            if (service == null || service.trim().length() == 0) {
                //throw new HeroHookException("The activity property 'service' is not definded or empty.");
                System.out.println("The activity property 'service' is not definded or empty.");
                return;
            }
            if (operation == null || operation.trim().length() == 0) {
                //throw new HeroHookException("The activity property 'operation' is not definded or empty.");
                System.out.println("The activity property 'operation' is not definded or empty.");
                return;
            }
            
            msgExchange.setMethod(MessageExchangeBean.INOUT_METHOD);
            msgExchange.setService(service);
            msgExchange.setOperation(operation);
            msgExchange.setInContent(getContentMessage("", activity.getBnProject().getName(), activity.getName(), PetalsHook.State.TERMINATED, projectProperties, activityProperties));
            
            MessageExchangeBean msgExchangeReturned = RmiServiceClientFactory.getRmiServiceClient().send(msgExchange);
            
            // get out msg
            String outContent = msgExchangeReturned.getOutContent();
            
            // parse out msg to get new activity and project prop values
            Map<String, String> newProjectProperties = new HashMap<String, String>();
            Map<String, String> newActivityProperties = new HashMap<String, String>();
            
            Document outDocument = loadDocument(outContent);
            Node content = outDocument.getFirstChild();
            NodeList children = content.getChildNodes();
            for (int i = 0; i < children.getLength(); i++) {
                Node child = children.item(i);
                if (child.getNodeType() != Node.ELEMENT_NODE) {
                    continue;
                }
                
                if (child.getNodeName().equals("process")) {
                    NodeList processChildren = child.getChildNodes();
                    for (int y = 0; y < processChildren.getLength(); y++) {
                        Node processChild = processChildren.item(y);
                        if (processChild.getNodeType() != Node.ELEMENT_NODE) {
                            continue;
                        }
                        String value = createStringFromNodeList(processChild.getChildNodes());
                        newProjectProperties.put(processChild.getNodeName(), value);
                    }
                }
                
                else if (child.getNodeName().equals("activity")) {
                    NodeList activityChildren = child.getChildNodes();
                    for (int y = 0; y < activityChildren.getLength(); y++) {
                        Node activityChild = activityChildren.item(y);
                        if (activityChild.getNodeType() != Node.ELEMENT_NODE) {
                            continue;
                        }
                        String value = createStringFromNodeList(activityChild.getChildNodes());
                        newActivityProperties.put(activityChild.getNodeName(), value);
                    }
                }
            }
            
            // set new values on the project instance and on the node
            copyNewProjectValuesInWorkflow(activity.getBnProject().getBnProperties(), projectProperties, newProjectProperties);
            copyNewActivityValuesInWorkflow(activity.getBnProperties(), activityProperties, newActivityProperties);
        } catch (Exception e) {
            e.printStackTrace();
            throw new HeroHookException(e.getMessage());
        }
        
        System.out.println("afterTerminate end");
    }

    private static void copyNewProjectValuesInWorkflow(Collection<BnProjectPropertyLocal> propertiesLocal, Map<String, String> oldProperties, Map<String, String> newProperties) {
        if (propertiesLocal != null) {
            for (BnProjectPropertyLocal property : propertiesLocal) {
                String key = property.getTheKey();
                if (newProperties.containsKey(key) && !newProperties.get(key).equals(oldProperties.get(key))) {
                    property.setTheValue(newProperties.get(key));
                }
            }
        }
    }

    private static void copyNewActivityValuesInWorkflow(Collection<BnNodePropertyLocal> propertiesLocal, Map<String, String> oldProperties, Map<String, String> newProperties) {
        if (propertiesLocal != null) {
            for (BnNodePropertyLocal property : propertiesLocal) {
                String key = property.getTheKey();
                if (newProperties.containsKey(key) && !newProperties.get(key).equals(oldProperties.get(key))) {
                    property.setTheValue(newProperties.get(key));
                }
            }
        }
    }

    /**
     * Load a document from an input stream.
     *
     * @param inputStream
     * @return
     * @throws IOException
     * @throws SAXException
     */
    public static Document loadDocument(String xml) throws IOException, SAXException {
        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
        docBuilderFactory.setNamespaceAware(true);

        DocumentBuilder docBuilder = null;
        try {
            docBuilder = docBuilderFactory.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
        }

        Document document = docBuilder.parse(new InputSource(new StringReader(xml)));
        
        return document;
    }

    /**
     * Create a String result from DOM NodeList
     *
     * @param nodes
     *            the DOM Nodes. Must not be null
     * @return a String representation of the DOM Document
     * @throws TransformerException
     */
    public static String createStringFromNodeList(NodeList nodes) throws TransformerException {
        final StringWriter out = new StringWriter();
        final Result resultStream = new StreamResult(out);
        final TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer;
        transformer = tFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        
        for (int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
            node.normalize();
            final Source source = new DOMSource(node);
            transformer.transform(source, resultStream);
        }
        
        return out.toString();
    }

    public void afterStart(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void anticipate(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void beforeStart(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void beforeTerminate(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void onCancel(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void onDeadline(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void onReady(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

}
