package hero.hook;

import hero.interfaces.BnNodeLocal;
import hero.interfaces.Constants;
import hero.util.HeroHookException;

import java.util.Map;

import org.objectweb.petals.binding.rmi.client.RmiServiceClientFactory;
import org.objectweb.petals.binding.rmi.model.MessageExchangeBean;

public class PetalsManualHook extends PetalsHook implements NodeHookI {

    public String getMetadata() {
        return Constants.Nd.AFTERTERMINATE;
    }

    @SuppressWarnings("unchecked")
    public void afterTerminate(Object engineSession, BnNodeLocal activity) throws HeroHookException {
        System.out.println("afterTerminate");
        
        try {
            Map<String, String> projectProps = getProjectPropertiesMap(activity.getBnProject().getBnProperties());
            Map<String, String> activityProps = getActivityPropertiesMap(activity.getBnProperties());

            String service = getService(activityProps);
            String operation = getOperation(activityProps);
            
            if (service == null || service.trim().length() == 0) {
                //throw new HeroHookException("The activity property 'service' is not definded or empty.");
                System.out.println("The activity property 'service' is not definded or empty.");
                return;
            }
            if (operation == null || operation.trim().length() == 0) {
                //throw new HeroHookException("The activity property 'operation' is not definded or empty.");
                System.out.println("The activity property 'operation' is not definded or empty.");
                return;
            }
            
            MessageExchangeBean msgExchange = new MessageExchangeBean();
            msgExchange.setService(service);
            msgExchange.setOperation(operation);
            msgExchange.setInContent(getContentMessage("", activity.getBnProject().getName(), activity.getName(), PetalsHook.State.TERMINATED, projectProps, activityProps));
            
            RmiServiceClientFactory.getRmiServiceClient().send(msgExchange);
        } catch (Exception e) {
            e.printStackTrace();
            throw new HeroHookException(e.getMessage());
        }
    }

    @SuppressWarnings("unchecked")
    public void onReady(Object engineSession, BnNodeLocal activity) throws HeroHookException {
        System.out.println("onReady");
        
        try {
            // TODO if it is a manual activity, get params and props and call manualActivityReady with them
//            String activityPerformer = nodeLocal.getActivityPerformer();
//            if (activityPerformer == null) {
//                // no performer => automatic activity => nothing to do
//                return;
//            }
            
            Map<String, String> projectProps = getProjectPropertiesMap(activity.getBnProject().getBnProperties());
            Map<String, String> activityProps = getActivityPropertiesMap(activity.getBnProperties());

            String service = getService(activityProps);
            String operation = getOperation(activityProps);
            String method = getMethod(activityProps);
            
            if (service == null || service.trim().length() == 0) {
                //throw new HeroHookException("The activity property 'service' is not definded or empty.");
                System.out.println("The activity property 'service' is not definded or empty.");
                return;
            }
            if (operation == null || operation.trim().length() == 0) {
                //throw new HeroHookException("The activity property 'operation' is not definded or empty.");
                System.out.println("The activity property 'operation' is not definded or empty.");
                return;
            }
            
            MessageExchangeBean msgExchange = new MessageExchangeBean();
            msgExchange.setService(service);
            msgExchange.setOperation(operation);
            msgExchange.setInContent(getContentMessage("", activity.getBnProject().getName(), activity.getName(), PetalsHook.State.READY, projectProps, activityProps));
            
            if (method != null && method.length() > 0) {
                msgExchange.setMethod(method);
            }
            
            RmiServiceClientFactory.getRmiServiceClient().send(msgExchange);
        } catch (Exception e) {
            e.printStackTrace();
            throw new HeroHookException(e.getMessage());
        }
    }
    
    public void afterStart(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void anticipate(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void beforeStart(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void beforeTerminate(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void onCancel(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

    public void onDeadline(Object arg0, BnNodeLocal arg1) throws HeroHookException {}

}
